/*
 *  Pythoong 
 * (C) 2000 christophe cressend
 *
 * 
 */

#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <gnome.h>
#include <glib.h>
#include <gdk_imlib.h>
#include <gdk/gdktypes.h>
#include <gdk/gdk.h>

#include <config.h>

#include "pythoong_def.h"
#include "graphics.h"

#define line_arrow_width SQUARE_SIZE
#define line_arrow_height SQUARE_SIZE

static GdkColor  *pythoong_bgcolor= NULL,*pythoong_fgcolor=NULL;
static GdkImage  *maze_image=NULL;
static GdkPixmap *pythoong_pixmap=NULL,*buf_pixmap=NULL,*title_pixmap=NULL,*icon_pixmap=NULL;
static GdkColormap * colormap=NULL;
static GdkVisual * visual= NULL;
static GdkGC * headgc=NULL,*bodygc=NULL,*joingc=NULL;

PythGC * PytGraphics_CreatePythGC(GtkWidget* app,GtkWidget  *win)
{
     PythGC* graphs = g_new(PythGC,1);
     graphs->widget=win;
     graphs->app=app;
     return (graphs);
}

void  PytGraphics_FreePythGC(PythGC* graphs)
{
     g_free(graphs);
}

void     PytGraphics_flush()
{
     gdk_flush();
}
gboolean PythGraphics_free()
{
     gint i;
     if(pythoong_pixmap != NULL )
     {
	  g_free(pythoong_pixmap);
	  pythoong_pixmap = NULL;
     }

     if(buf_pixmap !=NULL)
     {
	  g_free(buf_pixmap);
	  buf_pixmap=NULL;
     }
     if (title_pixmap !=NULL)
     {
	  g_free(title_pixmap);
	  title_pixmap=NULL;
     }
     if ( bodygc) gdk_gc_unref (bodygc);
     if ( headgc) gdk_gc_unref (headgc);     
     if ( joingc) gdk_gc_unref (joingc);
     if (pythoong_bgcolor) g_free(pythoong_bgcolor);
     if (pythoong_fgcolor) g_free(pythoong_fgcolor);
    
     return TRUE;
}

void PytGraphics_draw_tile(gint tile,
	       gint  x,
	       gint  y,
	       PythGC* graphs)
{
     if((tile < 0) || (tile > PIX_MAXTILES))
     {
	  g_print("Overflow in draw_tile : blank");
	  gdk_draw_pixmap(buf_pixmap, graphs->widget->style->black_gc,
			  pythoong_pixmap, BLANK*SQUARE_SIZE, 
			  0, x*SQUARE_SIZE, y*SQUARE_SIZE, SQUARE_SIZE, SQUARE_SIZE);
     } 
     else 
     {
	  gdk_draw_pixmap(buf_pixmap, graphs->widget->style->black_gc,
			  pythoong_pixmap, tile*SQUARE_SIZE, 
			  0, x*SQUARE_SIZE, y*SQUARE_SIZE, SQUARE_SIZE, SQUARE_SIZE);
     }
}

void PytGraphics_draw_tile_true(gint tile,
	       gint  x,
	       gint  y,
	       PythGC* graphs)
{
     if((tile < 0) || (tile > PIX_MAXTILES))
     {
	  g_print("Overflow in draw_tile : blank");
	  gdk_draw_pixmap(buf_pixmap, graphs->widget->style->black_gc,
			  pythoong_pixmap, BLANK*SQUARE_SIZE, 
			  0, x, y, SQUARE_SIZE, SQUARE_SIZE);
     } 
     else 
     {
	  gdk_draw_pixmap(buf_pixmap, graphs->widget->style->black_gc,
			  pythoong_pixmap, tile*SQUARE_SIZE, 
			  0, x, y, SQUARE_SIZE, SQUARE_SIZE);
     }
}

void PytGraphics_put_title(PythGC* graphs)
{
//     gdk_window_clear_area(buf_pixmap,OFFSET_TITLE_X,OFFSET_TITLE_Y,PIX_TITLE_X,PIX_TITLE_Y);
     gdk_draw_pixmap(buf_pixmap, graphs->widget->style->black_gc,
     	     title_pixmap, 0,0,OFFSET_TITLE_X,OFFSET_TITLE_Y,PIX_TITLE_X,PIX_TITLE_Y);
     PytGraphics_send_local_buf(graphs,OFFSET_TITLE_X,OFFSET_TITLE_Y,PIX_TITLE_X,PIX_TITLE_Y);
}

void PytGraphics_next_level(PythGC* graphs,gint level)
{
     gdk_window_clear_area(graphs->widget->window,0,0,GAME_SIZE_X,GAME_SIZE_Y);
     gdk_flush();
}

void PytGraphics_clear_win(PythGC* graphs)
{
  gdk_window_clear_area(graphs->widget->window, 0, 0, GAME_SIZE_X, GAME_SIZE_Y);
}


void PytGraphics_send_buf(PythGC* graphs)
{
	  gdk_draw_pixmap(graphs->widget->window, graphs->widget->style->black_gc,
			  buf_pixmap, 0,0,0,0,GAME_SIZE_X,GAME_SIZE_Y);
    
}

void PytGraphics_send_local_buf(PythGC* graphs,gint x,gint y,gint width,gint height)
{
   gdk_draw_pixmap (graphs->widget->window, graphs->widget->style->fg_gc[GTK_WIDGET_STATE
			    (graphs->widget)], buf_pixmap,x,y,x,y,width,height);
//   gdk_draw_rectangle	 (graphs->widget->window,graphs->widget->style->fg_gc[GTK_WIDGET_STATE(graphs->widget)],0,x,y,width,height);// funny   
}
void  PytGraphics_init_graphics()
{
     visual = gdk_imlib_get_visual();
     gtk_widget_push_visual (visual);

     colormap=gdk_imlib_get_colormap();
     gtk_widget_push_colormap (colormap);

     if(visual->type != GDK_VISUAL_TRUE_COLOR)
     {
          gdk_imlib_set_render_type(RT_PLAIN_PALETTE);
     }
}

gboolean PytGraphics_load_graphics(PythGC* graphs,Prefs* prefs)
{
     gint i;
     gboolean bo;
     gulong background;
     gchar *file=NULL;
     GString *f=NULL;
     GdkImlibImage* title_img=NULL;
     GdkImage *tmpimage=NULL,*tmptitle=NULL;

     if (NULL !=title_pixmap)
     {
          g_free(title_pixmap);
          title_pixmap=NULL;
     }

     if (NULL !=buf_pixmap)
     {
          g_free(buf_pixmap);
          buf_pixmap=NULL;
     }

     // Load title pixmap.
     f=g_string_new(PYT_OTH_DIR);
     g_string_append(f,TITLE);
     file = gnome_unconditional_pixmap_file (f->str);
     g_string_free(f,TRUE);
     if (!g_file_exists (file))
     {
          g_print ("File %s does not exist.\n",file);
          return (FALSE);
     }
     title_img = gdk_imlib_load_image(file);
     gdk_imlib_render(title_img, title_img->rgb_width, title_img->rgb_height);
     title_pixmap = gdk_pixmap_new (graphs->widget->window,title_img->rgb_width, title_img->rgb_height,-1);
     title_pixmap = gdk_imlib_move_image(title_img);
     gdk_imlib_destroy_image(title_img);

     // Load icon pixmap ( re-use title structs).
     f=g_string_new(PYT_OTH_DIR);
     g_string_append(f,ICON);
     file = gnome_unconditional_pixmap_file (f->str);
     g_string_free(f,TRUE);
     if (!g_file_exists (file))
     {
          g_print ("File %s does not exist.\n",file);
          return (FALSE);
     }
     title_img = gdk_imlib_load_image(file);
     gdk_imlib_render(title_img, title_img->rgb_width, title_img->rgb_height);
     icon_pixmap = gdk_pixmap_new (graphs->widget->window,title_img->rgb_width, title_img->rgb_height,-1);
     icon_pixmap = gdk_imlib_move_image(title_img);
     gdk_imlib_destroy_image(title_img);
     
     // Set icon
     gdk_window_set_icon(graphs->app->window, NULL, icon_pixmap, NULL);

     // Initialize buf pixmap
     buf_pixmap = gdk_pixmap_new (graphs->widget->window,GAME_SIZE_X,GAME_SIZE_Y,-1);

     return TRUE;
}

gboolean PytGraphics_load_snake(PythGC* graphs,gchar* pix)
{
     gint i;
     gchar *file=NULL;
     GString *f=NULL;
     GdkImlibImage* pythoong_img=NULL;
     GdkImage *tmpimage;

     if (NULL != pythoong_pixmap)
     {
	  g_free(pythoong_pixmap);
          pythoong_pixmap=NULL;
     }
     f=g_string_new(PYT_SNA_DIR);
     g_string_append(f,pix);
     g_string_append (f,".png");

     file = gnome_unconditional_pixmap_file (f->str);
     g_string_free(f,TRUE);     
     if (!g_file_exists (file))
     {
          g_print ("File %s does not exist.\n",file);
          return (FALSE);
     }
     // load image and render to pixmap
     pythoong_img = gdk_imlib_load_image(file);
     gdk_imlib_render(pythoong_img, pythoong_img->rgb_width, pythoong_img->rgb_height);
     pythoong_pixmap = gdk_imlib_move_image(pythoong_img);
     gdk_imlib_destroy_image(pythoong_img);

     // get bg color
     tmpimage = gdk_image_get(pythoong_pixmap, 0, 0, 1, 1);
     if (! pythoong_bgcolor) pythoong_bgcolor=g_new(GdkColor,1);
     pythoong_bgcolor->pixel = gdk_image_get_pixel(tmpimage, 0, 0);

     if (! pythoong_fgcolor) pythoong_fgcolor=g_new(GdkColor,1);
     pythoong_fgcolor->pixel = gdk_image_get_pixel(tmpimage, SQUARE_SIZE/2, SQUARE_SIZE/2);  //middle of the head

     gdk_window_set_background(graphs->widget->window, pythoong_bgcolor);

}

gboolean PytGraphics_load_maze_image(gchar* filename)
{
     gchar* gnfilename;
     GdkImlibImage* img=NULL;
     GdkPixmap *tmpbmp=NULL;
     GString *f1;

     f1=g_string_new(PYT_MAZ_DIR);
     g_string_append(f1,filename);
     g_string_append (f1,".xpm");
     gnfilename = gnome_unconditional_pixmap_file (f1->str);
     if (!g_file_exists (gnfilename))
     {
  	    g_string_free(f1,TRUE);
  	    return (FALSE);
     }
     img = gdk_imlib_load_image(gnfilename);
     g_string_free(f1,TRUE);
     
     if (GAME_SIZE_TILE_X != img->rgb_width)
     {
	  g_print ("Bad X image size : %d should be %d",img->rgb_width,GAME_SIZE_TILE_X);
	  return (FALSE);
     }
     
     if (GAME_SIZE_TILE_Y != img->rgb_height)
     {
	  g_print ("Bad Y image size : %d should be %d",img->rgb_height,GAME_SIZE_TILE_Y);
	  return (FALSE);
     }
     if (NULL == img)
     {
	  g_print ("Couldn't load bitmap file %s !\n",filename);
	  return (FALSE);
     }
     gdk_imlib_render(img, img->rgb_width,img->rgb_height);
     tmpbmp = gdk_imlib_move_image(img);
     
     if (NULL == tmpbmp)
     {
	  g_print ("Couldn't load bitmap file %s !\n",filename);
	  return (FALSE);
     }
     
     maze_image = gdk_image_get(tmpbmp, 0, 0, img->rgb_width, img->rgb_height);
     gdk_imlib_destroy_image(img);
     return TRUE;

}
void PytGraphics_calc_collision(gint tilehead,gint tilebody,PythGC* graphs)
{
     gint i;
     GdkColor *tmp_col;
     GdkGC *tmp_gc;
     GdkGCValues val;
     GdkPixmap *tempo_pixmap;

     
     gdk_draw_pixmap(pythoong_pixmap,graphs->widget->style->black_gc,
		     pythoong_pixmap, (MASKS+tilehead)*SQUARE_SIZE, 
		     0, COLLISION*SQUARE_SIZE,0, SQUARE_SIZE, SQUARE_SIZE);

     gdk_draw_pixmap(pythoong_pixmap,graphs->widget->style->black_gc,
		     pythoong_pixmap, (MASKS+tilehead)*SQUARE_SIZE, 
		     0, (COLLISION+1)*SQUARE_SIZE,0, SQUARE_SIZE, SQUARE_SIZE);

     if (!bodygc)
     {
	  val.fill=GDK_TILED;
	  val.function=GDK_AND;
	  bodygc =gdk_gc_new_with_values (pythoong_pixmap,&val,GDK_GC_FILL| GDK_GC_FUNCTION); 

	  val.function=GDK_AND_REVERSE;
	  headgc =gdk_gc_new_with_values (pythoong_pixmap,&val,GDK_GC_FILL| GDK_GC_FUNCTION); 
     
	  val.function=GDK_OR;
	  joingc =gdk_gc_new_with_values (pythoong_pixmap,&val,GDK_GC_FILL| GDK_GC_FUNCTION);  
     }
     gdk_draw_pixmap(pythoong_pixmap,bodygc,
		     pythoong_pixmap, tilebody*SQUARE_SIZE,0,COLLISION*SQUARE_SIZE,0,SQUARE_SIZE,SQUARE_SIZE);

     gdk_draw_pixmap(pythoong_pixmap,headgc,
		     pythoong_pixmap, tilehead*SQUARE_SIZE,0,(COLLISION+1)*SQUARE_SIZE,0,SQUARE_SIZE,SQUARE_SIZE);
     gdk_draw_pixmap(pythoong_pixmap,joingc,
		     pythoong_pixmap,(COLLISION+1)*SQUARE_SIZE,0,COLLISION*SQUARE_SIZE,0,SQUARE_SIZE,SQUARE_SIZE);     

}
gint PytGraphics_get_maze(gint x,gint y)
{ 
     
     gint val=gdk_image_get_pixel(maze_image,x,y);
     if (val) return 0; else return 1;
}

/*********************************************************************/
/*Thats' all folk                                         */
/*********************************************************************/
