/*
 *  Pythoong 
 * (C) 2000 christophe cressend
 *
 * 
 */
#include <dirent.h>
#include <gdk/gdkkeysyms.h>

#include "gnome.h"
#include "pythoong_def.h"
#include "prefs.h"
#include "keyboard.h"
#include "tgmath.h"

extern void reinit_speed();
extern void reinit_graphics();

typedef struct _stringval
{
     gchar* str;
     const gint  val;
}
stringval;

typedef struct _checkstruct
{
     gchar* name;
     gboolean checked;
}
Check;

typedef struct _checkstruct2
{
     gchar* file;
     gint   level;
     gboolean checked;
}
CheckLevel;

#define DEFAULT_PIX "standard"
#define DEFAULT_NUMBER 3;
#define DEFAULT_SPEED 50;
#define DEFAULT_SPEED_STR _(FAST);
#define DEFAULT_LEVEL 100;
#define DEFAULT_LEVEL_STR _(HARD);
#define DEFAULT_AUTO_MOVE TRUE;
#define DEFAULT_SIMPLE_GRAPHICS FALSE;
#define DEFAULT_CHANGE_SNAKE TRUE;
#define DEFAULT_RANDOM_MAZE  FALSE;
#define DEFAULT_DIAMOND 1000;

static const gint DEF_KEY[]={GDK_Up,GDK_Left,GDK_Down,GDK_Right};

static const stringval PrefLevels[]={{_(CHILD_LEVEL),20},
    	       			   {_(VERY_EASY),40},
    	       			   {_(EASY),60},
    	       			   {_(MIDDLE),80},
    	       			   {_(HARD),100},
    	       			   {_(VERY_HARD),120},
     	       			   {_(NIGHTMARE),140},
               			   {NULL,0}};

static const stringval PrefSpeeds[]={{_(CHILD_SPEED),150},
    	       			   {_(VERY_SLOW),110},
    	       			   {_(SLOW),90},
    	       			   {_(MIDDLE),70},
    	       			   {_(FAST),50},
    	       			   {_(VERY_FAST),30},
     	       			   {_(NIGHTMARE),20},
               			   {NULL,0}};

static const stringval PrefDiamond[]={{_(NEVER),0},
                                      {_(ALMOST_NEVER),1000},
				      {_(RARELY),300},
				      {_(SOMETIMES),100},
				      {_(OFTEN),30},
				      {_(VERY_OFTEN),10},
				      {_(ALWAYS),3},
				      {NULL,-1}};

static GtkWidget* _box=NULL;
static Prefs * _Ptempo=NULL; 
static Prefs * _Pval=NULL;
static GList * ListMazes;
static GList * ListSnakes;


// Callbacks
static void free_widget (GtkWidget *widget, void *data)
{
//	free (data);
}

static void check_simple_snake_graphics_clb(GtkWidget *widget, gpointer data)
{
     _Ptempo->simple_graphics=GTK_TOGGLE_BUTTON(widget)->active;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}
static void check_change_snake_clb(GtkWidget *widget, gpointer data)
{
     _Ptempo->change_snake=GTK_TOGGLE_BUTTON(widget)->active;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}

static void check_auto_move_clb(GtkWidget *widget, gpointer data)
{
     _Ptempo->auto_move=GTK_TOGGLE_BUTTON(widget)->active;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}

static void check_random_maze_clb(GtkWidget *widget, gpointer data)
{
     _Ptempo->random_maze=GTK_TOGGLE_BUTTON(widget)->active;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}

static void select_scenario (GtkWidget *widget, void *data)
{
     PytPref_set_pixmap(_Ptempo,(gchar*) data);
     _Ptempo->pix_changed=TRUE;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}

static gint get_int_level_value(gchar * level)
{
     gint i=0;
     gint val=-1;
     if (level == NULL) return -1;
     while (PrefLevels[i].val != 0)
     {
	  if (g_strcasecmp(level,PrefLevels[i].str) ==0)
	  {
	       val=PrefLevels[i].val;
	  }
          i++;
     }
     if ( val == -1) val=DEFAULT_LEVEL;
     return val;
}
gchar* get_char_level_value(gint level)
{
     gint i=0;
     gchar * val=NULL;
     while (PrefLevels[i].val != 0)
     {
	  if (level == PrefLevels[i].val)
	  {
	       val=PrefLevels[i].str;
	  }
          i++;
     }
     if (val == NULL) val=DEFAULT_LEVEL_STR;
     return val;
}

static gint get_int_speed_value(gchar * speed)
{
     gint i=0;
     gint val=-1;
     if (speed == NULL ) return -1;
     while (PrefSpeeds[i].val != 0)
     {
	  if (g_strcasecmp(speed,PrefSpeeds[i].str) ==0)
	  {
	       val=PrefSpeeds[i].val;
	  }
          i++;
     }
     if (val==-1) val=DEFAULT_SPEED;
     return val;
}

static gint get_int_diamond_freq_value(gchar * speed)
{
     gint i=0;
     gint val=-1;
     if (speed == NULL ) return -1;
     while (PrefDiamond[i].val != -1)
     {
	  if (g_strcasecmp(speed,PrefDiamond[i].str) ==0)
	  {
	       val=PrefDiamond[i].val;
	  }
          i++;
     }
     if (val==-1) val=DEFAULT_DIAMOND;
     return val;
}

gchar* get_char_speed_value(gint speed)
{
     gint i=0;
     gchar * val=NULL;
     while (PrefSpeeds[i].val != 0)
     {
	  if (speed == PrefSpeeds[i].val)
	  {
	       val=PrefSpeeds[i].str;
	  }
          i++;
     }
     if (val == NULL) val=DEFAULT_SPEED_STR;
     return val;
}

static void select_level (GtkWidget *widget, void *data)
{
     _Ptempo->level = get_int_level_value((gchar *) data);
     if (_Ptempo->level ==-1) _Ptempo->level=DEFAULT_LEVEL;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}
static void select_speed (GtkWidget *widget, void *data)
{
     _Ptempo->speed=get_int_speed_value((gchar*) data);
     if (_Ptempo->speed ==-1) _Ptempo->speed=DEFAULT_SPEED;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}

static void select_diamond_freq (GtkWidget *widget, void *data)
{
     _Ptempo->diamond_freq=get_int_diamond_freq_value((gchar*) data);
     if (_Ptempo->diamond_freq ==-1) _Ptempo->diamond_freq=DEFAULT_DIAMOND;
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}

static void nb_changed_clb (GtkWidget *widget, gpointer data)
{
        _Ptempo->number_orig=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(data));  
	gnome_property_box_changed (GNOME_PROPERTY_BOX (_box));
}

static void keypad_clb(GtkWidget*widget,GdkEventKey *event,gpointer     data)
{
     gint val,num;
     val = event->keyval;
     num = (gint)data;
     val = PytKey_keyupcase(val);
     _Ptempo->dir[num]=val;
     gtk_entry_set_text(GTK_ENTRY(widget),PytKey_getkeystring(val));
     gnome_property_box_changed(GNOME_PROPERTY_BOX(_box));
}

static void destroy_clb  (GtkWidget *widget, void *data)
{
     g_free(_Ptempo);
     _Ptempo=NULL;
     _Pval=NULL;
     _box=NULL;
}
static void apply_clb (GtkWidget *widget, void *data)
{
     PytPref_copy_to(_Ptempo,_Pval);
     if (_Ptempo->pix_changed)
     {
          reinit_graphics();            
     }
     reinit_speed();
     PytPref_calc_score_increase(_Pval);
     _Ptempo->pix_changed=FALSE;
}

static void fill_pixmenu(Prefs *prefs,GtkWidget* menu)
{
     GtkWidget* item;
     GList *tmp=ListSnakes;
     gint number=g_list_length(ListSnakes),i=0;
     while (i < number)
     {
	  gchar * value=g_strdup(((Check*)tmp->data)->name);
	  item = gtk_menu_item_new_with_label (((Check*) tmp->data)->name);
	  gtk_widget_show (item);
	  gtk_menu_append (GTK_MENU(menu), item);
	  gtk_signal_connect (GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(select_scenario),value);
	  if (!g_strcasecmp(prefs->pythoon_pixmap, ((Check*)tmp->data)->name))
	  {
	       gtk_menu_set_active(GTK_MENU(menu), i);
	  }
	  i++;
          tmp=tmp->next;
     }
}

static void fill_levelmenu(Prefs *prefs,GtkWidget* menu)
{
     GtkWidget *item; 
     gint i=0;
     while (PrefLevels[i].val != 0)
     {
	  gchar * value=g_strdup(PrefLevels[i].str);
	  item = gtk_menu_item_new_with_label (PrefLevels[i].str);
	  gtk_widget_show (item);
	  gtk_menu_append (GTK_MENU(menu), item);
	  gtk_signal_connect (GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(select_level), value);
	  if (prefs->level==PrefLevels[i].val )
	  {
	       gtk_menu_set_active(GTK_MENU(menu), i);
	  }
	  i++;
     }
}

static void fill_speedmenu(Prefs *prefs,GtkWidget* menu)
{
     GtkWidget *item;
     gint i=0;
     while (PrefSpeeds[i].val != 0)
     {
	  gchar * value=g_strdup(PrefSpeeds[i].str);
	  item = gtk_menu_item_new_with_label (PrefSpeeds[i].str);
	  gtk_widget_show (item);
	  gtk_menu_append (GTK_MENU(menu), item);
	  gtk_signal_connect (GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(select_speed), value);
	  if (prefs->speed==PrefSpeeds[i].val )
	  {
	       gtk_menu_set_active(GTK_MENU(menu), i);
	  }
	  i++;
     }
}

static void fill_diamond_freq_menu(Prefs *prefs,GtkWidget* menu)
{
     GtkWidget *item;
     gint i=0;
     while (PrefDiamond[i].val != -1)
     {
	  gchar * value=g_strdup(PrefDiamond[i].str);
	  item = gtk_menu_item_new_with_label (PrefDiamond[i].str);
	  gtk_widget_show (item);
	  gtk_menu_append (GTK_MENU(menu), item);
	  gtk_signal_connect (GTK_OBJECT(item), "activate", GTK_SIGNAL_FUNC(select_diamond_freq), value);
	  if (prefs->diamond_freq==PrefDiamond[i].val )
	  {
	       gtk_menu_set_active(GTK_MENU(menu), i);
	  }
	  i++;
     }
}

void  PytPref_show_box(Prefs * prefs,gint game_state,GtkWidget* app)
{
     Prefs* tmpprefs;
     GtkWidget *p1,*p2;  //2 pages
     GtkWidget *valuelabel,*keylabel;             
     GtkWidget *llevel,*lspeed,*ltype,*lpix,*lab;             
     GtkWidget *vb1,*vb2,*vb3,*vb4,*vb5;
     GtkWidget *hb1,*hb2,*hb3,*hb4,*hb5,*hb6;
     GtkWidget *f1,*f2,*f3,*f4,*f5,*f6;
     GtkWidget *b1,*b2,*b3,*b4;
     GtkWidget *tmenulevel,*menulevel,*tmenuspeed,*menuspeed,*tmenupix,*menupix,*tmenudiam,*menudiam;
     GtkWidget *se,*lnum;
     GtkObject *adj;
     gint rc;
     GtkWidget *table;
     GtkWidget *entkey[4];

     if (_box) return;
     
     if (_Ptempo != NULL) g_free(_Ptempo); // never know !
     _Ptempo=g_new0(Prefs,1);
     PytPref_copy_to(prefs,_Ptempo);
     _Pval=prefs;
     _Ptempo->pix_changed=FALSE;

     _box = gnome_property_box_new();

     rc = gtk_signal_connect (GTK_OBJECT (_box), "destroy",GTK_SIGNAL_FUNC(destroy_clb),NULL);
     rc = gtk_signal_connect (GTK_OBJECT (_box), "apply"  ,GTK_SIGNAL_FUNC(apply_clb),NULL);
  
     gtk_window_set_title(GTK_WINDOW(&GNOME_PROPERTY_BOX(_box)->dialog.window),
			  _("Pythoong preferences"));

     // Define first page: values
     p1 = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (p1);
     gtk_container_set_border_width(GTK_CONTAINER(p1), GNOME_PAD_SMALL);

     vb1 = gtk_vbox_new (TRUE, GNOME_PAD_SMALL);
     gtk_container_border_width (GTK_CONTAINER (vb1), GNOME_PAD_SMALL);
     gtk_widget_show (vb1);
     gtk_box_pack_start(GTK_BOX(p1), vb1, FALSE, TRUE, GNOME_PAD_SMALL);

     // Difficulty
     f1 = gtk_frame_new (_ ("Difficulty level"));
     gtk_container_border_width (GTK_CONTAINER (f1), 5);
     gtk_box_pack_start_defaults (GTK_BOX(vb1), f1);	
     gtk_widget_show (f1);

     hb1 = gtk_hbox_new(TRUE, GNOME_PAD_SMALL);
     gtk_widget_show (hb1);
     gtk_container_add (GTK_CONTAINER (f1), hb1);

     if (game_state == STATE_PLAYING)
     {
	  gchar * lab=(_("Unknown"));
	  gint i=0;
	  while (PrefLevels[i].val != 0)
	  {
	       if (_Ptempo->level == PrefLevels[i].val)
	       {
		    lab=PrefLevels[i].str;
	       }
	       i++;
	  }
	  lnum = gtk_label_new(lab);
	  gtk_widget_show (lnum);
	  gtk_box_pack_start_defaults(GTK_BOX (hb1), lnum);
     }
     else
     {
	  tmenulevel = gtk_option_menu_new();
	  menulevel = gtk_menu_new();
	  fill_levelmenu(_Ptempo,menulevel);
	  gtk_widget_show(tmenulevel);
	  gtk_option_menu_set_menu(GTK_OPTION_MENU(tmenulevel),menulevel);
	  gtk_box_pack_start_defaults(GTK_BOX(hb1), tmenulevel);
     }
     
     //Speed
     f2 = gtk_frame_new (_("Speed"));
     gtk_container_border_width (GTK_CONTAINER (f2), 5);
     gtk_box_pack_start_defaults (GTK_BOX(vb1), f2);	
     gtk_widget_show (f2);

     hb2 = gtk_hbox_new(TRUE, GNOME_PAD_SMALL);
     gtk_widget_show (hb2);
     gtk_container_add (GTK_CONTAINER (f2), hb2);

     if (game_state == STATE_PLAYING)
     {
	  gchar * lab=(_("Unknown"));
	  gint i=0;
	  while (PrefSpeeds[i].val != 0)
	  {
	       if (_Ptempo->speed == PrefSpeeds[i].val)
	       {
		    lab=PrefSpeeds[i].str;
	       }
	       i++;
	  }
	  lnum = gtk_label_new(lab);
	  gtk_widget_show (lnum);
	  gtk_box_pack_start_defaults(GTK_BOX (hb2), lnum);
     }
     else
     {
	  tmenuspeed = gtk_option_menu_new();
	  menuspeed = gtk_menu_new();
	  fill_speedmenu(_Ptempo,menuspeed);
	  gtk_widget_show(tmenuspeed);
	  gtk_option_menu_set_menu(GTK_OPTION_MENU(tmenuspeed),menuspeed);
	  gtk_box_pack_start_defaults(GTK_BOX(hb2), tmenuspeed);
     }
     
     // Snake parameters
     f3 = gtk_frame_new (_("Snakes pixmap & Life Number :"));
     gtk_container_border_width (GTK_CONTAINER (f3), 5);
     gtk_box_pack_start_defaults (GTK_BOX(vb1), f3);	
     gtk_widget_show (f3);

     hb3 = gtk_hbox_new(TRUE, GNOME_PAD_SMALL);
     gtk_widget_show (hb3);
     gtk_container_add (GTK_CONTAINER (f3), hb3);

     tmenupix = gtk_option_menu_new();
     menupix = gtk_menu_new();
     fill_pixmenu(_Ptempo,menupix);
     gtk_widget_show(tmenupix);
     gtk_option_menu_set_menu(GTK_OPTION_MENU(tmenupix),menupix);
     gtk_box_pack_start_defaults(GTK_BOX(hb3), tmenupix);

     adj = gtk_adjustment_new(_Ptempo->number_orig,1,20,1,5,10);
     se = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 10, 0);
     gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(se),
				       GTK_UPDATE_ALWAYS);
     gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(se), 1);
     gtk_signal_connect (GTK_OBJECT (adj), "value_changed", GTK_SIGNAL_FUNC
			 (nb_changed_clb), se);
     gtk_box_pack_start (GTK_BOX (hb3), se, TRUE, TRUE, 0);
     gtk_widget_show(se);

     // Diamond freq popup
     f6 = gtk_frame_new (_("Diamond"));
     gtk_container_border_width (GTK_CONTAINER (f6), 5);
     gtk_box_pack_start_defaults (GTK_BOX(vb1), f6);	
     gtk_widget_show (f6);

     hb6 = gtk_hbox_new(TRUE, GNOME_PAD_SMALL);
     gtk_widget_show (hb6);
     gtk_container_add (GTK_CONTAINER (f6), hb6);

     if (game_state == STATE_PLAYING)
     {
	  gchar * lab=(_("Unknown"));
	  gint i=0;
	  while (PrefDiamond[i].val != 0)
	  {
	       if (_Ptempo->diamond_freq == PrefDiamond[i].val)
	       {
		    lab=PrefDiamond[i].str;
	       }
	       i++;
	  }
	  lnum = gtk_label_new(lab);
	  gtk_widget_show (lnum);
	  gtk_box_pack_start_defaults(GTK_BOX (hb6), lnum);
     }
     else
     {
	  tmenudiam = gtk_option_menu_new();
	  menudiam = gtk_menu_new();
	  fill_diamond_freq_menu(_Ptempo,menudiam);
	  gtk_widget_show(tmenudiam);
	  gtk_option_menu_set_menu(GTK_OPTION_MENU(tmenudiam),menudiam);
	  gtk_box_pack_start_defaults(GTK_BOX(hb6), tmenudiam);
     }


     valuelabel = gtk_label_new(_("Game"));
     gtk_notebook_append_page(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(_box)->notebook), 
			      p1, valuelabel);

     /* The control page */
     p2 = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
     gtk_container_set_border_width(GTK_CONTAINER(p2), GNOME_PAD_SMALL);
     gtk_widget_show (p2);

     hb4 = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
     gtk_widget_show (hb4);
     gtk_box_pack_start_defaults (GTK_BOX(p2), hb4);	

     f4 = gtk_frame_new (_("Keys :"));
     gtk_container_border_width (GTK_CONTAINER (f4), 5);
     gtk_widget_show (f4);
     gtk_box_pack_start_defaults (GTK_BOX(hb4), f4);	

     // Left part : keyboard commands
     table = gtk_table_new(5, 5, TRUE);
     gtk_table_set_row_spacings(GTK_TABLE(table), 20);
     gtk_widget_show(table);
     gtk_container_add (GTK_CONTAINER (f4), table);

     lab = gtk_label_new(_("N"));
     gtk_table_attach_defaults(GTK_TABLE(table), lab, 2, 3, 0, 1);
     gtk_widget_show(lab);
     entkey[0] = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(entkey[0]), PytKey_getkeystring(_Ptempo->dir[0]));
     gtk_entry_set_editable(GTK_ENTRY(entkey[0]), FALSE);
     gtk_widget_set_usize(entkey[0], KB_WIDTH, KB_HEIGHT);
     gtk_table_attach_defaults(GTK_TABLE(table), entkey[0], 2 ,3, 1, 2);
     gtk_widget_show(entkey[0]);
     gtk_signal_connect(GTK_OBJECT(entkey[0]), "key_press_event",
			GTK_SIGNAL_FUNC(keypad_clb), (gpointer)0);

     lab = gtk_label_new(_("O"));
     gtk_table_attach_defaults(GTK_TABLE(table), lab, 0, 1, 2, 3);
     gtk_widget_show(lab);
     entkey[1] = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(entkey[1]), PytKey_getkeystring(_Ptempo->dir[1]));
     gtk_entry_set_editable(GTK_ENTRY(entkey[1]), FALSE);
     gtk_widget_set_usize(entkey[1], KB_WIDTH, KB_HEIGHT);
     gtk_table_attach_defaults(GTK_TABLE(table), entkey[1], 1, 2, 2,3);
     gtk_widget_show(entkey[1]);
     gtk_signal_connect(GTK_OBJECT(entkey[1]), "key_press_event",
			GTK_SIGNAL_FUNC(keypad_clb), (gpointer)1);

     lab = gtk_label_new(_("S"));
     gtk_table_attach_defaults(GTK_TABLE(table), lab, 2, 3, 4, 5);
     gtk_widget_show(lab);
     entkey[2] = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(entkey[2]), PytKey_getkeystring(_Ptempo->dir[3]));
     gtk_entry_set_editable(GTK_ENTRY(entkey[2]), FALSE);
     gtk_widget_set_usize(entkey[2], KB_WIDTH, KB_HEIGHT);
     gtk_table_attach_defaults(GTK_TABLE(table), entkey[2], 3, 4, 2, 3);
     gtk_widget_show(entkey[2]);
     gtk_signal_connect(GTK_OBJECT(entkey[2]), "key_press_event",
			GTK_SIGNAL_FUNC(keypad_clb), (gpointer)3);

     lab = gtk_label_new(_("E"));
     gtk_table_attach_defaults(GTK_TABLE(table), lab, 4, 5, 2, 3);
     gtk_widget_show(lab);
     entkey[3] = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(entkey[3]), PytKey_getkeystring(_Ptempo->dir[2]));
     gtk_entry_set_editable(GTK_ENTRY(entkey[3]), FALSE);
     gtk_widget_set_usize(entkey[3], KB_WIDTH, KB_HEIGHT);
     gtk_table_attach_defaults(GTK_TABLE(table), entkey[3], 2, 3, 3, 4);
     gtk_widget_show(entkey[3]);
     gtk_signal_connect(GTK_OBJECT(entkey[3]), "key_press_event",
			GTK_SIGNAL_FUNC(keypad_clb), (gpointer)2);

     // Right part : games parameters
     f5 = gtk_frame_new (_("Controls :"));
     gtk_container_border_width (GTK_CONTAINER (f5), 5);
     gtk_widget_show (f5);
     gtk_box_pack_start_defaults (GTK_BOX(hb4), f5);	

     vb5 =gtk_vbox_new(TRUE, GNOME_PAD_SMALL);
     gtk_widget_show (vb5);
     gtk_container_add (GTK_CONTAINER (f5), vb5);

     b1 = gtk_check_button_new_with_label(_("Simple snake graphics"));
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b1),_Ptempo->simple_graphics);
     gtk_signal_connect (GTK_OBJECT (b1), "toggled",GTK_SIGNAL_FUNC (check_simple_snake_graphics_clb),NULL);
     gtk_box_pack_start (GTK_BOX (vb5), b1, FALSE, FALSE, 0);
     gtk_widget_show(b1);


     if (game_state == STATE_PLAYING)
     {
	  if ( _Ptempo->change_snake)
	  { 
	       lab = gtk_label_new(_("Change snake at each level"));
	  }
	  else
	  {
               lab = gtk_label_new(_("Keep the same snake"));
	  }
	  gtk_box_pack_start_defaults (GTK_BOX(vb5), lab);
	  gtk_widget_show(lab);
     }
     else
     {
	  b2 = gtk_check_button_new_with_label(_("Change graphics  at each level"));
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b2),_Ptempo->change_snake);
	  gtk_signal_connect (GTK_OBJECT (b2), "toggled",GTK_SIGNAL_FUNC (check_change_snake_clb),NULL);
	  gtk_box_pack_start (GTK_BOX (vb5), b2, FALSE, FALSE, 0);
	  gtk_widget_show(b2);
     }
     
     if (game_state == STATE_PLAYING)
     {
	  if ( _Ptempo->auto_move)
	  { 
	       lab = gtk_label_new(_("Mazes in random order"));
	  }
	  else
	  {
	       lab = gtk_label_new(_("Ordered mazes"));
	  }
	  gtk_box_pack_start_defaults (GTK_BOX(vb5), lab);
	  gtk_widget_show(lab);
     }
     else
     {

	  b3 = gtk_check_button_new_with_label(_("Mazes in random order"));
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b3),_Ptempo->random_maze);
	  gtk_signal_connect (GTK_OBJECT (b3), "toggled",GTK_SIGNAL_FUNC (check_random_maze_clb),NULL);
	  gtk_box_pack_start (GTK_BOX (vb5), b3, FALSE, FALSE, 0);
	  gtk_widget_show(b3);
     }
     if (game_state == STATE_PLAYING)
     {
	  if ( _Ptempo->auto_move)
	  { 
	       lab = gtk_label_new(_("Auto-move selected"));
	  }
	  else
	  {
               lab = gtk_label_new(_("Auto-move non selected"));
	  }
	  gtk_box_pack_start_defaults (GTK_BOX(vb5), lab);
	  gtk_widget_show(lab);
     }
     else
     {
	  b4 = gtk_check_button_new_with_label(_("Auto-move on corners"));
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (b4),_Ptempo->auto_move);
	  gtk_signal_connect (GTK_OBJECT (b4), "toggled",GTK_SIGNAL_FUNC (check_auto_move_clb),NULL);
	  gtk_box_pack_start (GTK_BOX (vb5), b4, FALSE, FALSE, 0);
	  gtk_widget_show(b4);
     }
     
     
     keylabel = gtk_label_new(_("Controls"));
     gtk_notebook_append_page(GTK_NOTEBOOK(GNOME_PROPERTY_BOX(_box)->notebook), 
			      p2, keylabel);

     gtk_widget_show(_box);
}
void PytPref_create_check_structs()
{
     DIR *dir;
     struct dirent *direntity;
     gint num=1;
     gchar *dname = gnome_unconditional_pixmap_file (PYT_SNA_DIR);
     dir = opendir (dname);
     if (!dir)
     {
            g_print("Cannot open directory %s",dname);
	    return;
     }
	  
     while ((direntity = readdir (dir)) != NULL)
     {
	    gchar *s = g_strdup (direntity->d_name);
	    if (strstr (direntity->d_name, ".png"))
	    {          
		      
		 gchar ** name=g_strsplit(s,".",1);
		 Check * check= g_new(Check,1);
		 check->name=g_strdup(name[0]);
		 check->checked=FALSE;
		 ListSnakes=g_list_append(ListSnakes,check);
		 g_strfreev(name);
	    }
     }
     closedir (dir);
     dname = gnome_unconditional_pixmap_file (PYT_MAZ_DIR);
     dir = opendir (dname);
     if (!dir)
     {
	    g_print("Cannot open directory %s",dname);
	    return;
     }
    
     while ((direntity = readdir (dir)) != NULL)
     {
	    gchar *s = g_strdup (direntity->d_name);
	    if (strstr (direntity->d_name, ".xpm"))
	    {          
		      
		 gchar ** name=g_strsplit(s,".",1);
		 CheckLevel * check= g_new(CheckLevel,1);
		 check->file=g_strdup(name[0]);
		 check->checked=FALSE;
		 check->level=num;
                 num+=1;
		 ListMazes=g_list_append(ListMazes,check);
		 g_strfreev(name);
	    }
     }
     closedir (dir);
}

void PytPref_set_pixmap(Prefs* prefs,gchar * newpix)
{
     if (prefs->pythoon_pixmap) g_free(prefs->pythoon_pixmap);
     prefs->pythoon_pixmap=g_new(gchar,strlen(newpix)+1);
     strcpy(prefs->pythoon_pixmap,newpix);
}

void  PytPref_load_prefs(Prefs*prefs)
{
     gint i;
     prefs->pythoon_pixmap= gnome_config_get_string_with_default("/pythoong/Preferences/Pixmap",NULL);
     if (prefs->pythoon_pixmap == NULL)
     {
	  PytPref_set_pixmap(prefs,DEFAULT_PIX);
     }
     prefs->dir[0]= PytKey_getkeyfromstring(gnome_config_get_string_with_default("/pythoong/Preferences/KeyNorth",NULL));
     prefs->dir[1]= PytKey_getkeyfromstring(gnome_config_get_string_with_default("/pythoong/Preferences/KeyOuest",NULL));       
     prefs->dir[2]= PytKey_getkeyfromstring(gnome_config_get_string_with_default("/pythoong/Preferences/KeySouth",NULL));       
     prefs->dir[3]= PytKey_getkeyfromstring(gnome_config_get_string_with_default("/pythoong/Preferences/KeyEst",NULL));         
     prefs->number_orig= gnome_config_get_int_with_default("/pythoong/Preferences/Number",NULL);         
     if (prefs->number_orig == 0) prefs->number_orig=DEFAULT_NUMBER;
     prefs->diamond_freq= gnome_config_get_int_with_default("/pythoong/Preferences/DiamFreq",NULL);         
     if (prefs->diamond_freq == 0) prefs->diamond_freq=DEFAULT_DIAMOND;
     prefs->speed= get_int_speed_value(gnome_config_get_string_with_default("/pythoong/Preferences/Speed",NULL));
     if (prefs->speed ==-1) prefs->speed=DEFAULT_SPEED;
     prefs->level= get_int_level_value(gnome_config_get_string_with_default("/pythoong/Preferences/Level",NULL));     
     if (prefs->level ==-1) prefs->level=DEFAULT_LEVEL;
     prefs->auto_move= gnome_config_get_bool_with_default("/pythoong/Preferences/AutoMove",NULL);
     prefs->change_snake= gnome_config_get_bool_with_default("/pythoong/Preferences/ChangeSnake",NULL);
     prefs->simple_graphics= gnome_config_get_bool_with_default("/pythoong/Preferences/SimpleGraphs",NULL);
     prefs->random_maze= gnome_config_get_bool_with_default("/pythoong/Preferences/RandomMaze",NULL);
     gnome_config_sync();

     for (i=0;i<4;i++)
     {
	  if (prefs->dir[i]==-1)
	  {
               gint j;
               for (j=0;j<4;j++) prefs->dir[j]=DEF_KEY[j];
               break;	       
	  }
     }
     PytPref_calc_score_increase(prefs);
}

void PytPref_save_prefs(Prefs*prefs)
{
     gnome_config_set_string("/pythoong/Preferences/Pixmap",prefs->pythoon_pixmap); 
     gnome_config_set_string("/pythoong/Preferences/KeyNorth", PytKey_getkeystring(prefs->dir[0]));
     gnome_config_set_string("/pythoong/Preferences/KeyOuest", PytKey_getkeystring(prefs->dir[1]));
     gnome_config_set_string("/pythoong/Preferences/KeySouth", PytKey_getkeystring(prefs->dir[2]));
     gnome_config_set_string("/pythoong/Preferences/KeyEst", PytKey_getkeystring(prefs->dir[3]));
     gnome_config_set_int("/pythoong/Preferences/Number", prefs->number_orig);
     gnome_config_set_int("/pythoong/Preferences/DiamFreq", prefs->diamond_freq);
     gnome_config_set_string("/pythoong/Preferences/Speed",get_char_speed_value(prefs->speed));
     gnome_config_set_string("/pythoong/Preferences/Level", get_char_level_value(prefs->level));
     gnome_config_set_bool("/pythoong/Preferences/AutoMove", prefs->auto_move);
     gnome_config_set_bool("/pythoong/Preferences/ChangeSnake", prefs->change_snake);
     gnome_config_set_bool("/pythoong/Preferences/SimpleGraphs", prefs->simple_graphics);
     gnome_config_set_bool("/pythoong/Preferences/RandomMaze", prefs->random_maze);  
     gnome_config_sync();
}


void PytPref_copy_to(Prefs * prefs1,Prefs* prefs2) 
{
     if ((prefs1 ==NULL) || (prefs2 ==NULL)) return;
     if (prefs2->pythoon_pixmap) g_free(prefs2->pythoon_pixmap);
     memcpy(prefs2, prefs1, sizeof(Prefs));
     prefs2->pythoon_pixmap=NULL;
     PytPref_set_pixmap(prefs2,prefs1->pythoon_pixmap);
}

void PytPref_calc_score_increase(Prefs*prefs)
{
     gint diam_val=(prefs->diamond_freq==0)?3000:prefs->diamond_freq;
     gfloat diam_param=log(diam_val);
     gint val=(((((prefs->level+400)*50*diam_param)/prefs->speed)/prefs->number_orig)*((prefs->auto_move==TRUE?2:1)));
     prefs->score_increase=val;
}

void PytPref_free(Prefs * prefs)
{
     if ( prefs )
     {
	  if (prefs->pythoon_pixmap)
	  {
	       g_free(prefs->pythoon_pixmap);
               g_free(prefs);
	  }
     }
     g_list_foreach (ListSnakes, (GFunc) g_free, NULL);
     g_list_foreach (ListMazes, (GFunc) g_free, NULL);

     g_list_free (ListSnakes);
     g_list_free (ListMazes);
}

gint Pyt_Prefs_get_next_level(Prefs* prefs)
{
     GList *tmp=ListMazes;
     gint loop=0,to_play=0;
     gint number=g_list_length(tmp);
     gboolean all_done=TRUE;
     while(loop <number)
     {
          if (((CheckLevel*)tmp->data)->checked == FALSE)
	  {
               all_done=FALSE;
               to_play++;
          }
	  tmp=tmp->next;
	  loop++;
     } 
     if (all_done) Pyt_Prefs_reinit_mazes();
     
     tmp=ListMazes;
     if (prefs->random_maze)
     {
          gint maze=(rand()%(to_play+1));
          while(maze>0)
          {
               do
               {
                    tmp=tmp->next;
                    if (tmp==NULL) tmp=ListMazes;
	       }
               while (((CheckLevel*)tmp->data)->checked == TRUE);
	       maze--;
	  }
     }
     else
     {
	  while (((CheckLevel*)tmp->data)->checked == TRUE)
	  {
	       tmp=tmp->next;
	       if (tmp==NULL) tmp=ListMazes;
	  }
     }
     ((CheckLevel*)tmp->data)->checked=TRUE;
     return (((CheckLevel*)tmp->data)->level);
}
gchar* Pyt_Prefs_get_next_pixmap()
{
     GList *tmp=ListSnakes;
     gint loop=0;
     gint number=g_list_length(tmp);
     gboolean all_done=TRUE;
     while(loop <number)
     {
          if (((Check*)tmp->data)->checked == FALSE)
	  {
               all_done=FALSE;
          }
	  tmp=tmp->next;
	  loop++;
     } 

     if (all_done)
     {
	  tmp=ListSnakes;
	  loop=0;
	  while(loop <number)
	  {
	       ((Check*)tmp->data)->checked = FALSE;
	       tmp=tmp->next;
	       loop++;
	  }
     }
     tmp=ListSnakes;
     while (((Check*)tmp->data)->checked == TRUE)
     {
	  tmp=tmp->next;
	  if (tmp==NULL) tmp=ListSnakes;
     }

     ((Check*)tmp->data)->checked=TRUE;
     return (((Check*)tmp->data)->name);
}

gchar* Pyt_Prefs_get_level_file(gint level)
{
     gint prevlevel=level;
     GList *tmp=ListMazes;
     tmp=ListMazes;
     while(level>1)
     {
	  tmp=tmp->next;
	  if (tmp==NULL) tmp=ListMazes;
	  level--;
     }     
     return (((CheckLevel*)tmp->data)->file);
}

void Pyt_Prefs_reinit_mazes()
{
     GList *tmp=ListMazes;
     gint number=g_list_length(tmp);
     gint loop=0;
     while(loop <number)
     {
          ((CheckLevel*)tmp->data)->checked = FALSE;
	  tmp=tmp->next;
	  loop++;
     }
}
