/**
   *  Pythoong 
   * (C) 2000 christophe cressend
   *
   * 
   **/

#include <stdlib.h>   
#include "gnome.h"
#include "pythoong.h"
#include "statusbar.h"
#include "snake.h"
#include "maze.h"
#include "prefs.h"
#include "keyboard.h"
#include "config.h"

//#define GAME_EVENTS   GDK_ALL_EVENTS_MASK	
#define GAME_EVENTS (  GDK_ENTER_NOTIFY_MASK          |\
		       GDK_LEAVE_NOTIFY_MASK          |\
		       GDK_KEY_PRESS_MASK	      |\
		       GDK_EXPOSURE_MASK)


// gnome stuff
static GtkWidget *app=NULL, *arena=NULL, *appbar=NULL;

static Snake* MySnake=NULL;
static Prefs* MyPrefs=NULL;
static PythGC* MyGraphics=NULL;
static gint game_state=STATE_DEMO;
static GTimer* tim;
static gint loop_id=0,loop_anim=0;
static gint keyb_id=0;

static gint score=0;
static gint remaining=0;
static gint level=0;
static gint game_level=0;
static gint game_pause=0;
static gint last=0,ydia=0,xdia=0;
static gboolean diamond=FALSE;
static gint diapix;

static gint menu_end_clb (GtkWidget *widget, gpointer data);
static gint game_new_cb (GtkWidget *widget, void *data);
static gint show_scores (GtkWidget *widget, void *data);
static gint game_quit_clb (GtkWidget *widget, void *data);
static gint ok_quit (GtkWidget *widget, void *data);
static gint ok_die (GtkWidget *widget, void *data);
static gint ok_quit_current (GtkWidget *widget, void *data);
static gint game_preferences_clb (GtkWidget *widget, void *data);
static gint save_preferences(GtkWidget *widget, void *data);
static gint game_about_clb (GtkWidget *widget, void *data);
static gint generate_move(Snake* snake,gint move,GtkWidget *widget);
static gint motion_notify_event_cb (GtkWidget *widget, GdkEventExpose *event);
static gint expose_event_cb (GtkWidget *widget, GdkEventExpose *event);
static gint loop_anim_clb(GtkWidget *widget, GdkEventKey *event);
static gint loop_clb(GtkWidget *widget, GdkEventKey *event);
static gint kbd_clb (GtkWidget *widget, GdkEventKey *event, gpointer data);
static void put_score();


GnomeUIInfo gamemenu[] = {
        GNOMEUIINFO_MENU_NEW_GAME_ITEM(game_new_cb, NULL),
	GNOMEUIINFO_SEPARATOR,
        GNOMEUIINFO_MENU_END_GAME_ITEM(menu_end_clb,NULL),
	GNOMEUIINFO_MENU_SCORES_ITEM(show_scores, NULL),
	GNOMEUIINFO_SEPARATOR,
        GNOMEUIINFO_MENU_EXIT_ITEM(game_quit_clb, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo settingsmenu[] = {
        GNOMEUIINFO_MENU_PREFERENCES_ITEM(game_preferences_clb, NULL),
	GNOMEUIINFO_ITEM_NONE("Save preferences",NULL,save_preferences),
        GNOMEUIINFO_END
};

GnomeUIInfo helpmenu[] = {
//        GNOMEUIINFO_HELP(GAME_NAME),
	GNOMEUIINFO_MENU_ABOUT_ITEM(game_about_clb, NULL),
	GNOMEUIINFO_END
};

GnomeUIInfo mainmenu[] = {
	GNOMEUIINFO_MENU_GAME_TREE(gamemenu),
	GNOMEUIINFO_MENU_SETTINGS_TREE(settingsmenu),
	GNOMEUIINFO_MENU_HELP_TREE(helpmenu),
	GNOMEUIINFO_END
};



static void start()
{
     if (!loop_id) loop_id=gtk_timeout_add (MyPrefs->speed,(GtkFunction) loop_clb,GTK_WIDGET(arena));
     if (!loop_anim) loop_anim=gtk_timeout_add (500,(GtkFunction) loop_anim_clb,GTK_WIDGET(arena));
}

static void stop()
{
     if (loop_id) gtk_timeout_remove (loop_id);
     if (loop_anim) gtk_timeout_remove (loop_anim);
     loop_id=0;
     loop_anim=0;
}

static gint menu_end_clb (GtkWidget *widget, gpointer data)
{
       gtk_widget_set_sensitive (gamemenu[2].widget, FALSE);
       init_computer_mode(MySnake,MySnake->maze,MyPrefs);
}

// Callbacks
static gint kbd_clb (GtkWidget *widget, GdkEventKey *event, gpointer data)
{
     gint i;
     gint val=event->keyval;
     if (GDK_Escape == event->keyval)
     {
	  init_computer_mode(MySnake,MySnake->maze,MyPrefs);                   
     }

     if (GNOME_KEY_NAME_PAUSE_GAME == event->keyval)
     {
	  game_pause=(game_pause+1)%2;
          if (game_pause) stop(); else start();
     }

     val=PytKey_keyupcase(val);
     for ( i=0;i<4;i++)
     {
	  if (val==MyPrefs->dir[i])
	  {
	       PytKey_push_key(i);
	  }
     }	
}

static gint expose_event_cb (GtkWidget *widget, GdkEventExpose *event)
{
     PytGraphics_send_local_buf(MyGraphics,event->area.x,event->area.y,event->area.width,event->area.height);
     return TRUE;
}

static gint game_about_clb (GtkWidget *widget, void *data)
{ 
	GtkWidget *about;
	const gchar *myhead[] = {
		"Christophe Cressend",
                " ",
                "Please send comments / bug at e-mail :",
                "christophe.cressend@wanadoo.fr",
                " ",
                "Pythoong pixmap / mazes  under GPL",
		NULL
	};

	about = gnome_about_new (_("Pythoong"), VERSION,
				 "(C) 2000 the Free Software Foundation",
				 (const char **) myhead,
				 _("Another snake game for Gnome..."),
				 "pythoong-icon.xpm");  
	gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(app));
	gtk_widget_show (about);

	return TRUE;
}


static gint die (GnomeClient *client, gpointer client_data)
{
     if (game_state == STATE_PLAYING)
     {
          GtkWidget *modbox;
          stop();
	  modbox = gnome_message_box_new(_("Do you really want to quit the current game and exit ?"),
					 GNOME_MESSAGE_BOX_QUESTION,
					 GNOME_STOCK_BUTTON_YES,
					 GNOME_STOCK_BUTTON_NO,
					 NULL);
	  gnome_dialog_set_parent(GNOME_DIALOG(modbox), GTK_WINDOW(app));
	  gnome_dialog_set_default(GNOME_DIALOG(modbox), 0);
	  gtk_window_set_modal(GTK_WINDOW(modbox), TRUE);
	  gtk_signal_connect(GTK_OBJECT(modbox), "clicked",GTK_SIGNAL_FUNC(ok_die), NULL);
	  gtk_widget_show(modbox);
     }
     else
     {
	  pythoong_free();
	  return FALSE;
     }
}

static gint save_yourself (GnomeClient *client,
			   gint phase,
			   GnomeSaveStyle save_style,
			   gint shutdown, 
			   GnomeInteractStyle interact_style,
			   gint fast,
			   gpointer client_data)
{
     // Nothing to save so far !  
     return TRUE;
}
static gint new_game()
{
     score=0;
     level=0;
     game_level=1;
     remaining=MyPrefs->number_orig;

     Pyt_Prefs_reinit_mazes();
     level=Pyt_Prefs_get_next_level(MyPrefs);
     PytMaze_init(level,MySnake->maze);
     PytMaze_generate_pills(MySnake->maze);
     init_board(MySnake,MySnake->maze,MyPrefs);
     PytGraphics_send_buf(MyGraphics);
     pythoong_statusbar_set(score,game_level,MySnake->maze->rem_pills,remaining);
     keyb_id=gtk_signal_connect (GTK_OBJECT (app),"key_press_event",GTK_SIGNAL_FUNC (kbd_clb), NULL);
     game_state=STATE_PLAYING;
     gtk_widget_set_sensitive (gamemenu[2].widget, TRUE);
     start();
     return(0);
}

static gint ok_die (GtkWidget *widget, void *data)
{
     gint button = GPOINTER_TO_INT(data);
     if(button == 0)
     {
	  pythoong_free();
	  return(TRUE);     
     }
     start();
     return(FALSE);     
}

static gint ok_quit_current (GtkWidget *widget, void *data)
{
     gint button = GPOINTER_TO_INT(data);
     game_state=STATE_DEMO;
     gtk_widget_set_sensitive (gamemenu[2].widget, FALSE);
     if(button != 0)
     {
          start();
	  return(FALSE);
     }
     new_game();
     return(TRUE);     
}

static gint game_new_cb (GtkWidget *widget, void *data)
{
     if (game_state == STATE_PLAYING)
     {
          GtkWidget *modbox;
          stop();
	  modbox = gnome_message_box_new(_("Do you really want to quit the current game ?"),
					 GNOME_MESSAGE_BOX_QUESTION,
					 GNOME_STOCK_BUTTON_YES,
					 GNOME_STOCK_BUTTON_NO,
					 NULL);
	  gnome_dialog_set_parent(GNOME_DIALOG(modbox), GTK_WINDOW(app));
	  gnome_dialog_set_default(GNOME_DIALOG(modbox), 0);
	  gtk_window_set_modal(GTK_WINDOW(modbox), TRUE);
	  gtk_signal_connect(GTK_OBJECT(modbox), "clicked",GTK_SIGNAL_FUNC(ok_quit_current), NULL);
	  gtk_widget_show(modbox);
     }
     else
     new_game();
     return(TRUE);
}
static void computer_mode_level_finished()
{
     stop();
     PytMaze_generate_pills(MySnake->maze); 
     if (MyPrefs->change_snake)
     { 
	  gchar* pix=Pyt_Prefs_get_next_pixmap();
	  PytGraphics_load_snake(MyGraphics,pix);
     }
     PytMaze_draw_maze(MySnake->maze,MyGraphics);
     PytMaze_draw_pills(MySnake->maze,MyGraphics);
     PytSnake_draw_init(MyPrefs->simple_graphics,MySnake,MyGraphics);
     PytGraphics_send_buf(MyGraphics);
     start();     
}


static gint show_scores (GtkWidget *widget, void *data)
{
      gnome_scores_display("Gnome Pythoong Scores",GAME_NAME,NULL, 0);
}
static gint game_quit_clb (GtkWidget *widget, void *data)
{
     GtkWidget *modbox;
     modbox = gnome_message_box_new(_("Do you really want to quit the current game ?"),
				    GNOME_MESSAGE_BOX_QUESTION,
				    GNOME_STOCK_BUTTON_YES,
				    GNOME_STOCK_BUTTON_NO,
				    NULL);
     gnome_dialog_set_parent(GNOME_DIALOG(modbox), GTK_WINDOW(app));
     gnome_dialog_set_default(GNOME_DIALOG(modbox), 0);
     gtk_window_set_modal(GTK_WINDOW(modbox), TRUE);
     gtk_signal_connect(GTK_OBJECT(modbox), "clicked",GTK_SIGNAL_FUNC(ok_quit), NULL);
     gtk_widget_show(modbox);
}
static gint ok_quit (GtkWidget *widget, void *data)
{
     gint button = GPOINTER_TO_INT(data);
     if(button != 0)
     {
	  return(1);
     }
     pythoong_free();
     gtk_main_quit();
}


static gint game_preferences_clb (GtkWidget *widget, void *data)
{
     PytPref_show_box(MyPrefs,game_state,app);
     return TRUE;     
}

static gint save_preferences (GtkWidget *widget, void *data)
{
     PytPref_save_prefs(MyPrefs);
     return TRUE;     
}

static gint check_diamond()
{
     //Diamond management
     if (diamond==FALSE)
     {
          if (MySnake->pills_since_diamond >10)
	  {
	       gint ran=rand()%MyPrefs->diamond_freq;
	       if (ran==1)
	       {
		    gint i,j;
		    gint num=rand()%MySnake->maze->rem_pills+1;
		    xdia=0;ydia=0;
		    diamond=TRUE;
		    last=rand()%100+100;
		    for (i=0;i<GAME_SIZE_TILE_X;i++)
		    {
			 for (j=0;j<GAME_SIZE_TILE_Y;j++)
			 {
			      if(MySnake->maze->pills[i][j]!=0)
			      {
				   num--; 
				   if (num==0)
				   {
					xdia=i; 
					ydia=j;
				   }
			      }
			 }
		    }
		    PytGraphics_draw_tile(DIAMOND1,xdia,ydia,MyGraphics);              
		    diapix=DIAMOND1;
		    PytGraphics_send_local_buf(MyGraphics,xdia*SQUARE_SIZE,ydia*SQUARE_SIZE,SQUARE_SIZE, SQUARE_SIZE);
	       }
	  }
     }	
}
static gint generate_move(Snake* snake,gint move,GtkWidget *widget)
{
     gint ret=PytSnake_move(snake,move);
     if ( ret  == -1) return (-1);
     PytSnake_draw(snake,MyGraphics,MyPrefs->simple_graphics);
     if (PytSnake_cannibalism(MySnake)) // eat my queue ?
     {
          stop();
          PytSnake_destroy(MySnake,MyGraphics);
          if (game_state !=STATE_DEMO)
	  { 
	       remaining=remaining-1;
               g_print("Remaining : %d\n",remaining);
               if (remaining == 0 )
	       {
                    g_print("init computer mode\n");
		    init_computer_mode(MySnake,MySnake->maze,MyPrefs);                   
 	            put_score();
	       }
	  }
          PytSnake_init(MySnake->maze,MySnake,4);
          PytKey_flush_key();
          gdk_flush();
          start();
     }

     if (diamond)
     {
	  if ((MySnake->val[0].x==xdia) && (MySnake->val[0].y==ydia)) // eat the diamond ?
	  {
	       diamond=FALSE;
	       last=0;
	       xdia=0;
	       ydia=0;
	       PytSnake_shorten(MySnake,MyGraphics);
               snake->pills_since_diamond=0;
	  }
	  last-=1;
	  if (last == 0)
	  {
	       PytGraphics_draw_tile(PILL,xdia,ydia,MyGraphics);              
	       PytGraphics_send_local_buf(MyGraphics,xdia*SQUARE_SIZE,ydia*SQUARE_SIZE,SQUARE_SIZE, SQUARE_SIZE);
	       diamond=FALSE;
	  }
     }
     if (PytMaze_is_pill(MySnake->maze,MySnake->val[0].x,MySnake->val[0].y)) // eat a pill ?
     {
	  PytMaze_eat_pill(MySnake->maze,MySnake->val[0].x,MySnake->val[0].y);
          snake->pills_since_diamond+=1;
	  score=score+MyPrefs->score_increase;
          MySnake->toqueue=MySnake->toqueue+MyPrefs->level;
	  pythoong_statusbar_set(score,game_level,MySnake->maze->rem_pills,remaining);
          if ( MySnake->maze->rem_pills == 0)
	  {
               if (game_state ==STATE_DEMO)
               {
		    computer_mode_level_finished();
 	       }
	       else
	       {
		    next_level();
	       }
          }
          check_diamond();
     }

     if (MySnake->toqueue > 100) //new queue needed ?
     {
	  MySnake->toqueue=MySnake->toqueue-100;
	  MySnake->grow=TRUE;
     }
     return (ret);
}

static gint loop_anim_clb(GtkWidget *widget, GdkEventKey *event)
{
     if (diamond)
     {
	  if ( diapix==DIAMOND1) diapix=DIAMOND2; else diapix=DIAMOND1;
	  PytGraphics_draw_tile(diapix,xdia,ydia,MyGraphics);              
	  PytGraphics_send_local_buf(MyGraphics,xdia*SQUARE_SIZE,ydia*SQUARE_SIZE,SQUARE_SIZE, SQUARE_SIZE);
     }
}


static gint loop_clb(GtkWidget *widget, GdkEventKey *event)
{
     gulong micro;

     if (game_pause) return 0;
     g_timer_elapsed(tim,&micro);
     
     if ( micro < MyPrefs->speed )
     { 
	  return (TRUE); // loop_cld called more than once each Prefs->speed !!
     }

     g_timer_stop(tim);
     g_timer_reset(tim);
     g_timer_start(tim);

     if (game_state == STATE_WAITING) // flush
     {
	  return(TRUE); 
     }

     // computer mode
     if (game_state == STATE_DEMO) 
     {
	  gint moved=-1;
	  gint dir_move;
	  dir_move=rand()%4;
	  while( moved == -1)
	  {
	       moved=generate_move(MySnake,dir_move,widget);
	       dir_move=(dir_move+1)%4;
	  }
     }
     else // player mode
     { // game mode
	  gint key,move;
	  key=PytKey_check_key();
          if (key == -1 && MyPrefs->auto_move)
	  {
               key=PytSnake_find_one_move(MySnake);
	       move = generate_move(MySnake,key,widget);     
	  }
          else
	  {	  
	       move = generate_move(MySnake,key,widget);     
	       if ((move == key) || ((move ==-1) && (PytSnake_checkmove(MySnake,key))))
	       {
		    PytKey_pull_key();
	       }
	  }
     }
     return (TRUE);
}

static void put_score()
{
     gint pos =  gnome_score_log(score,NULL, TRUE);
     if (pos <=10)
     {
          g_print("Position :%d\n",pos);
	  show_scores(NULL,NULL);
     }
}
gint next_level()
{
     stop();
     game_level+=1;
     level=Pyt_Prefs_get_next_level(MyPrefs);
     PytGraphics_next_level(MyGraphics,level);
     if (MyPrefs->change_snake)
     { 
	  gchar* pix=Pyt_Prefs_get_next_pixmap();
          PytGraphics_load_snake(MyGraphics,pix);
     }
     
     PytMaze_init(level,MySnake->maze);
     init_board(MySnake,MySnake->maze,MyPrefs);
     PytGraphics_send_buf(MyGraphics);
     start();
}

void init_board(Snake* snake,Maze* maze,Prefs* prefs)
{
     if (snake && maze)
     {
	  PytSnake_init(maze,snake,4);
	  PytMaze_draw_maze(maze,MyGraphics);
	  PytMaze_draw_pills(maze,MyGraphics);
	  PytSnake_draw_init(prefs->simple_graphics,snake,MyGraphics);
     }
}

gint init_computer_mode(Snake* snake,Maze* maze,Prefs* prefs)
{ 
     level=1;
     game_level=1;
     if (FALSE == PytMaze_init(level,maze))
     {
	  return (1);
     }
     score=0;
     remaining=0;
     init_board(snake,maze,prefs);
     PytGraphics_send_buf(MyGraphics);
     pythoong_statusbar_set(score,game_level,MySnake->maze->rem_pills,remaining);
     game_state=STATE_DEMO;
     gtk_widget_set_sensitive (gamemenu[2].widget, FALSE);
     start();
     return(0);    
}
gint pythoong_free()
{
     if ( MySnake != NULL )
     {
	  if ( MySnake->maze != NULL)
	  {
	       g_free(MySnake->maze);
	  }
	  g_free(MySnake);         
     }

     if ( MyPrefs != NULL) 
     {
          PytPref_free(MyPrefs);
     }
  
     PytGraphics_FreePythGC(MyGraphics);
     gtk_exit(0);
}

// Called by prefs.c
void reinit_speed()
{
     stop();
     start();
}
void reinit_graphics()
{
     PytGraphics_load_snake(MyGraphics,MyPrefs->pythoon_pixmap);
     PytMaze_draw_maze(MySnake->maze,MyGraphics);
     PytMaze_draw_pills(MySnake->maze,MyGraphics);
     PytSnake_draw_init(MyPrefs->simple_graphics,MySnake,MyGraphics); 
     if (level ==1) PytGraphics_put_title(MyGraphics);
     PytGraphics_send_buf(MyGraphics);        
}

 gint main(int argc,
          char *argv[])
{
     GdkImlibImage *image;
     GdkPixmap *mazeimg, *pythoonimg;
     GnomeClient *client;
     Maze * MyMaze=NULL;
     gint           i;
     gint score;
     gint j;
     gint xpos,ypos;
     gint dir_move=0;
     gint loop_clb_int,kbd_clb_int;
     
     gtk_init(&argc,&argv);
     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     textdomain(PACKAGE);

     gnome_score_init(GAME_NAME);

     gnome_init_with_popt_table(GAME_NAME, VERSION, argc, argv, options, 0, NULL);

     client = gnome_master_client();

     gtk_object_ref(GTK_OBJECT(client));
     gtk_object_sink(GTK_OBJECT(client));

     gtk_signal_connect(GTK_OBJECT(client), "save_yourself",
			GTK_SIGNAL_FUNC(save_yourself), argv[0]);
     gtk_signal_connect(GTK_OBJECT(client), "die",
			GTK_SIGNAL_FUNC(die), argv[0]);

     srand (time (NULL));

     app = gnome_app_new(GAME_NAME, _("Pythoong") );

     gtk_signal_connect(GTK_OBJECT(app), "delete_event",GTK_SIGNAL_FUNC(die), NULL);     
     gtk_signal_connect(GTK_OBJECT(app), "destroy_event",GTK_SIGNAL_FUNC(die), NULL);     

     gtk_window_set_policy(GTK_WINDOW(app), FALSE, FALSE, TRUE);

     appbar = pythoong_statusbar_new();
     gnome_app_set_statusbar(GNOME_APP(app), appbar); 

     gnome_app_create_menus(GNOME_APP(app), mainmenu);

     gnome_app_install_menu_hints(GNOME_APP (app), mainmenu);
     arena = gtk_drawing_area_new();
     gnome_app_set_contents(GNOME_APP(app), arena);     
     gtk_drawing_area_size(GTK_DRAWING_AREA(arena), GAME_SIZE_X, GAME_SIZE_Y);
  
     gtk_widget_set_events (arena, GAME_EVENTS);
     gtk_widget_realize (arena);
     gtk_signal_connect (GTK_OBJECT(arena), "expose_event",GTK_SIGNAL_FUNC (expose_event_cb), NULL);

     MyGraphics = PytGraphics_CreatePythGC(app,arena);
     MyPrefs=g_new(Prefs,1);

     PytPref_load_prefs(MyPrefs);
     PytGraphics_init_graphics();            

     if (!PytGraphics_load_snake(MyGraphics,MyPrefs->pythoon_pixmap))
     {
	  g_print("Could not load graphics !");
          exit(EXIT_FAILURE);
     }

     if (FALSE == PytGraphics_load_graphics(MyGraphics,MyPrefs))
     {
	  g_print("Could not load graphics !");
          exit(EXIT_FAILURE);
     }
     PytPref_create_check_structs();
     MySnake=g_new(Snake,1);         
     MyMaze=g_new(Maze,1);
     MySnake->maze=MyMaze;
     init_computer_mode(MySnake,MyMaze,MyPrefs);
     gtk_widget_show(app);

     tim = g_timer_new();

     gtk_main();

     gtk_object_unref(GTK_OBJECT(client));
     pythoong_free();     
     gtk_exit(EXIT_FAILURE);
}
